#include "channelwindow.h"
#include "userprofilewindow.h"
#include "tracklistwindow.h"
#include "playlistlistwindow.h"
#include "grouplistwindow.h"
#include "userlistwindow.h"
#include <QTabBar>
#include <QStackedWidget>
#include <QVBoxLayout>

ChannelWindow::ChannelWindow(const QString &userId, QWidget *parent) :
    QWidget(parent),
    m_userId(userId),
    m_profileTab(new UserProfileWindow(userId, this)),
    m_tracksTab(0),
    m_favouritesTab(0),
    m_playlistsTab(0),
    m_groupsTab(0),
    m_followingsTab(0),
    m_tabBar(new QTabBar(this)),
    m_stack(new QStackedWidget(this)),
    m_vbox(new QVBoxLayout(this))
{
    this->setWindowTitle(tr("Channel"));
    this->setAttribute(Qt::WA_DeleteOnClose);

    m_tabBar->setDrawBase(false);
    m_tabBar->addTab(tr("Profile"));
    m_tabBar->addTab(tr("Tracks"));
    m_tabBar->addTab(tr("Favourites"));
    m_tabBar->addTab(tr("Sets"));
    m_tabBar->addTab(tr("Groups"));
    m_tabBar->addTab(tr("Followings"));

    m_stack->addWidget(m_profileTab);

    m_vbox->setContentsMargins(0, 0, 0, 0);
    m_vbox->setSpacing(0);
    m_vbox->addWidget(m_tabBar);
    m_vbox->addWidget(m_stack);

    this->connect(m_tabBar, SIGNAL(currentChanged(int)), this, SLOT(onCurrentTabChanged(int)));
}

ChannelWindow::ChannelWindow(QSharedPointer<UserItem> user, QWidget *parent) :
    QWidget(parent),
    m_userId(user.data()->id()),
    m_profileTab(new UserProfileWindow(user, this)),
    m_tracksTab(0),
    m_favouritesTab(0),
    m_playlistsTab(0),
    m_groupsTab(0),
    m_followingsTab(0),
    m_tabBar(new QTabBar(this)),
    m_stack(new QStackedWidget(this)),
    m_vbox(new QVBoxLayout(this))
{
    this->setWindowTitle(tr("Channel"));
    this->setAttribute(Qt::WA_DeleteOnClose);

    m_tabBar->setDrawBase(false);
    m_tabBar->addTab(tr("Profile"));
    m_tabBar->addTab(tr("Tracks"));
    m_tabBar->addTab(tr("Favourites"));
    m_tabBar->addTab(tr("Sets"));
    m_tabBar->addTab(tr("Groups"));
    m_tabBar->addTab(tr("Followings"));

    m_stack->addWidget(m_profileTab);

    m_vbox->setContentsMargins(0, 0, 0, 0);
    m_vbox->setSpacing(0);
    m_vbox->addWidget(m_tabBar);
    m_vbox->addWidget(m_stack);

    this->connect(m_tabBar, SIGNAL(currentChanged(int)), this, SLOT(onCurrentTabChanged(int)));
}

ChannelWindow::~ChannelWindow() {}

void ChannelWindow::showProfile() {
    m_stack->setCurrentWidget(m_profileTab);
    m_profileTab->setFocus(Qt::OtherFocusReason);
}

void ChannelWindow::showTracks() {
    if (!m_tracksTab) {
        m_tracksTab = new TrackListWindow(this);
        m_stack->addWidget(m_tracksTab);
        m_tracksTab->setFocus(Qt::OtherFocusReason);
        m_tracksTab->setTrackFeed(QUrl(QString("http://api.soundcloud.com/users/%1/tracks.json?limit=30").arg(m_userId)));
    }

    m_stack->setCurrentWidget(m_tracksTab);
    m_tracksTab->setFocus(Qt::OtherFocusReason);
}

void ChannelWindow::showFavourites() {
    if (!m_favouritesTab) {
        m_favouritesTab = new TrackListWindow(this);
        m_stack->addWidget(m_favouritesTab);
        m_favouritesTab->setFocus(Qt::OtherFocusReason);
        m_favouritesTab->setTrackFeed(QUrl(QString("http://api.soundcloud.com/users/%1/favorites.json?limit=30").arg(m_userId)));
    }

    m_stack->setCurrentWidget(m_favouritesTab);
    m_favouritesTab->setFocus(Qt::OtherFocusReason);
}

void ChannelWindow::showPlaylists() {
    if (!m_playlistsTab) {
        m_playlistsTab = new PlaylistListWindow(this);
        m_stack->addWidget(m_playlistsTab);
        m_playlistsTab->setFocus(Qt::OtherFocusReason);
        m_playlistsTab->setPlaylistFeed(QUrl(QString("http://api.soundcloud.com/users/%1/playlists.json?limit=30").arg(m_userId)));
    }

    m_stack->setCurrentWidget(m_playlistsTab);
    m_playlistsTab->setFocus(Qt::OtherFocusReason);
}

void ChannelWindow::showGroups() {
    if (!m_groupsTab) {
        m_groupsTab = new GroupListWindow(this);
        m_stack->addWidget(m_groupsTab);
        m_groupsTab->setFocus(Qt::OtherFocusReason);
        m_groupsTab->setGroupFeed(QUrl(QString("http://api.soundcloud.com/users/%1/groups.json?limit=30").arg(m_userId)));
    }

    m_stack->setCurrentWidget(m_groupsTab);
    m_groupsTab->setFocus(Qt::OtherFocusReason);
}

void ChannelWindow::showFollowings() {
    if (!m_followingsTab) {
        m_followingsTab = new UserListWindow(this);
        m_stack->addWidget(m_followingsTab);
        m_followingsTab->setFocus(Qt::OtherFocusReason);
        m_followingsTab->setUserFeed(QUrl(QString("http://api.soundcloud.com/users/%1/followings.json?limit=30").arg(m_userId)));
    }

    m_stack->setCurrentWidget(m_followingsTab);
    m_followingsTab->setFocus(Qt::OtherFocusReason);
}

void ChannelWindow::onCurrentTabChanged(int index) {
    switch (index) {
    case 1:
        this->showTracks();
        return;
    case 2:
        this->showFavourites();
        return;
    case 3:
        this->showPlaylists();
        return;
    case 4:
        this->showGroups();
        return;
    case 5:
        this->showFollowings();
        return;
    default:
        this->showProfile();
        return;
    }
}
