#include "userlisttab.h"
#include "userlistdelegate.h"
#include "tracklistwindow.h"
#include "channelwindow.h"
#include "windowstack.h"
#include "listview.h"
#include "nowplayingbutton.h"
#include "../base/userlistmodel.h"
#include "../base/notifications.h"
#include "../base/settings.h"
#include <QLabel>
#include <QGridLayout>
#include <QAction>
#include <QMaemo5Style>

UserListTab::UserListTab(QWidget *parent) :
    AbstractTab(parent),
    m_model(new UserListModel(this)),
    m_listView(new ListView(this)),
    m_noResultsLabel(new QLabel(QString("<p style='color:%1;font-size:40px;'><center><b>%2</b></center></p>").arg(QMaemo5Style::standardColor("SecondaryTextColor").name()).arg(tr("No followings found")), this)),
    m_grid(new QGridLayout(this))
{
    m_grid->setContentsMargins(0, 0, 0, 0);
    m_grid->addWidget(m_listView, 0, 0);
    m_listView->setModel(m_model);
    m_listView->setItemDelegate(new UserListDelegate(m_listView));
    m_listView->setMonitorScrolling(true);
    m_noResultsLabel->hide();

    this->connect(m_model, SIGNAL(loadingChanged(bool)), this, SLOT(onLoadingChanged(bool)));
    this->connect(m_model, SIGNAL(countChanged(int)), this, SLOT(onCountChanged(int)));
    this->connect(m_listView, SIGNAL(clicked(QModelIndex)), this, SLOT(onItemClicked(QModelIndex)));
    this->connect(m_listView, SIGNAL(atEnd()), m_model, SLOT(getMoreUsers()));

    if (Settings::instance()->mediaPlayer() == "musikloud") {
        this->connect(m_listView, SIGNAL(scrollingStarted()), NowPlayingButton::instance(), SLOT(hide()));
        this->connect(m_listView, SIGNAL(scrollingStopped()), NowPlayingButton::instance(), SLOT(poke()));
    }

    this->onOrientationChanged();
}

void UserListTab::setUserFeed(const QUrl &feed) {
    m_model->getUsers(feed);
}

void UserListTab::setLandscapeLayout() {
    m_noResultsLabel->setFixedSize(800, 400);
}

void UserListTab::setPortraitLayout() {
    m_noResultsLabel->setFixedSize(480, 700);
}

void UserListTab::onLoadingChanged(bool loading) {
    Notifications::toggleBusy(loading);
    m_noResultsLabel->setVisible((!loading) && (m_model->rowCount() == 0));
}

void UserListTab::onCountChanged(int count) {
    m_noResultsLabel->setVisible((count == 0) && (!m_model->loading()));
}

void UserListTab::onItemClicked(const QModelIndex &index) {
    ChannelWindow *window = new ChannelWindow(m_model->get(index.row()), WindowStack::instance()->currentWindow());
    window->show();
}
